###########################################################################
#
#   Copyright (c) 2000,2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000,2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.11 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Products
import Acquisition
import AccessControl.Role
import OFS.Folder
import calendar_day
import base_item
import time
import calendar
import utils

manage_add_calendar_month_form = HTMLFile('calendar_month_add', globals())


def manage_add_calendar_month(self, id, title, REQUEST=None):
	"""Add an instance of the calendar_month class."""

	self._setObject(id, calendar_month(id, title))

	if REQUEST is not None:

		return self.manage_main(self, REQUEST)


class calendar_month(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"""Used for grouping days."""

	use_local_navigation = 0

	meta_type = 'CalendarMonth'
	icon = 'misc_/CalendarFolder/Folder_icon.gif'

	_properties = (
		{'id':'title', 'type':'string'},
		)

	meta_types = (
		{'name':'CalendarDay', 'action':'manage_add_calendar_day_form'},
		)

	month_names = calendar.month_name
	month_names_abbreviated = calendar.month_abbr

	manage_add_calendar_day_form = calendar_day.manage_add_calendar_day_form
	manage_add_calendar_day = calendar_day.manage_add_calendar_day

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add CalendarDay',
		 ('manage_add_calendar_day', 'manage_add_calendar_day_form')),
		('View CalendarMonths',
		 ('',)),
		)

	index_html = HTMLFile('calendar_month_index', globals())
	calendar_month_display_listing = HTMLFile('calendar_month_display_listing', globals())


	def __init__(self, id, title=''):
		"Initialize the class instance."

		self.id = id
		self.title = title


	def render_name(self):
		"""Renders the month name."""

		return self.month_names_abbreviated[int(self.id)]


	def get_last_day(self):
		"""Returns the last day of the month."""

		return self.objectValues('CalendarDay')[-1]


	def get_first_day(self):
		"""Returns the first day of the month."""

		return self.objectValues('CalendarDay')[1]


	def get_previous_month(self):
		"""Returns the previous month."""

		if self.id == '01':

			# We need to look in the previous year

			previous_year = self.get_year().get_previous_year()
			previous_month = previous_year['12']

		else:

			previous_month = self.get_year()[utils.pad_integer(int(self.id) - 1)]

		return previous_month


	def get_next_month(self):
		"""Returns the next month."""

		if self.id == '12':

			# We need to look in the next year

			next_year = self.get_year().get_next_year()
			next_month = next_year['01']

		else:

			next_month = self.get_year()[utils.pad_integer(int(self.id) + 1)]

		return next_month


	def get_context_month(self):

		return self


	def get_context_month_id(self):

		return self.id


	def get_month_structure(self):
		"""Returns a structure describing the days of the current month."""

		return calendar.monthcalendar(int(self.getParentNode().id),
						int(self.id))


	def get_month(self):
		"""Returns self."""

		return self


	def get_days(self):
		"""Returns all days contained."""

		return self.objectValues('CalendarDay')


	def simple_navigation_html(self):
		"""Returns a simple month navigation list."""

		calendar_month_list = []
		calendar_months = self.get_months()

		for month in calendar_months:

			month_html = """&nbsp;<a href="%s/01/">%s</a>&nbsp;""" % \
					(month.relative_url(),
					month.render_name())

			if month.id == self.id:

				is_current_month = 1

			else:

				is_current_month = 0

			calendar_month_list.append({'html':month_html,'is_current_month':is_current_month})

		return calendar_month_list


	def get_week_days(self, day):
		"""Returns the week days surrounding the day."""

		year = int(self.get_year().id)
		month = int(self.id)

		for week_ in calendar.monthcalendar(year, month):

			if day in week_:

				week = week_

		object_week = []

		# If the week starts or ends in a previous or next month

		if 0 in week:

			# If the week starts in the previous month

			if week[0] == 0:

				number_of_days_in_previous_month = week.count(0)

				for day in week[number_of_days_in_previous_month:]:

					object_week.append(self[utils.pad_integer(day)])

				previous_month = self.get_previous_month()

				for day in range(1, number_of_days_in_previous_month + 1):

					object_week.append(previous_month.get_day_by_offset(-day))

			# If not, it must end in the next month

			else:

				number_of_days_in_next_month = week.count(0)

				for day in week[:-number_of_days_in_next_month]:

					assert day

					object_week.append(self[utils.pad_integer(day)])

				next_month = self.get_next_month()

				for day in range(1, number_of_days_in_next_month + 1):

					object_week.append(next_month.get_day_by_offset(day))

		else:

			for day in week:

				object_week.append(self[utils.pad_integer(day)])

		return object_week


	def get_day_by_offset(self, offset):
		"""Returns a day depending on the offset."""

		return self[self.objectIds()[offset]]


	def display(self, details=None, day=None):
		"""Returns an HTML formatted output of the current month."""

		tmp = """<table border="0" cellpadding="%s" cellspacing="%s"
		valign="top" bgcolor="%s" width="100%%">
		<tr><th colspan="7" bgcolor="%s">%s</th></tr>""" % \
			(self.table_cellpadding0,
			self.table_cellspacing0,
			self.color_header1,
			self.color_header_description0,
			self.title)


		tmp = tmp + """
		<tr>
		<td valign="middle"> Mon </td>
		<td valign="middle"> Tue </td>
		<td valign="middle"> Wed </td>
		<td valign="middle"> Thu </td>
		<td valign="middle"> Fri </td>
		<td valign="middle"> Sat </td>
		<td valign="middle"> Sun </td>
		</tr>"""


		if day:

			context_year = int(self.getParentNode().id)
			context_month = int(self.id)
			context_day = int(day)

		else:

			context_year, context_month, context_day = None, None, None

		is_current_year = 1
		is_current_month = 1

		weeks = self.get_month_structure()

		for week_index in range(len(weeks)):

			if week_index % 2:

				bgcolor = self.color_sequence_even0

			else:

				bgcolor = self.color_sequence_odd0

			tmp = tmp + """<tr bgcolor="%s">
			""" % bgcolor

			week = weeks[week_index]
				
			for day in week:

				if day:

					if details:

						object_count = len(self[utils.pad_integer(day)].objectIds())

						if object_count:

							formatted_object_count = " (%s)" % object_count

						else:

							formatted_object_count = ""

					else:

						formatted_object_count = ""


					if is_current_month and is_current_year and context_day == day:

						tmp = tmp + \
							"""<td align="center" bgcolor="#ffffff">""" + \
							"""<a href="%s">%s</a>%s</td>""" % \
								(self[utils.pad_integer(day)].absolute_url(),
								utils.pad_integer(day), formatted_object_count)

					else:

						tmp = tmp + \
							"""<td align="center"><a href="%s">%s</a>%s</td>""" % \
								(self[utils.pad_integer(day)].absolute_url(),
								 utils.pad_integer(day), formatted_object_count)

				else:

					tmp = tmp + """<td>&nbsp;</td>"""

			else:

				tmp = tmp + """</tr>"""

		tmp = tmp + """</table>"""

		return tmp


default__class_init__(calendar_month)

