###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.1 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import Products
import OFS.Folder
import base_item
import personal_file


personal_container_add_redirect = HTMLFile('personal_container_add_redirect', globals())
manage_add_personal_container_form = HTMLFile('personal_container_add', globals())


def manage_add_personal_container(self, name, description, REQUEST=None, RESPONSE=None):
	"Add an instance of the personal_container class"

	new_id = str(self.get_unique_id())

	self._setObject(new_id, personal_container(new_id, name, description))

	if RESPONSE is not None:

		RESPONSE.redirect(self.absolute_url())


class personal_container(
	AccessControl.Role.RoleManager,
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'PersonalContainer'
	icon = 'misc_/PersonalContainer/Folder_icon.gif'

	_properties = (
		{'id':'name', 'type':'string'},
		{'id':'description', 'type':'text'}
		)

	meta_types = (
		{'name':'PersonalContainer', 'action':'manage_add_personal_container_form'},
		{'name':'PersonalFile', 'action':'manage_add_personal_file_form'},
		)

	manage_add_personal_container = manage_add_personal_container
	manage_add_personal_file_form = personal_file.manage_add_personal_file_form
	manage_add_personal_file = personal_file.manage_add_personal_file

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View PersonalContainers',
		 ('',)),
		)

	#index_html = HTMLFile('personal_container_index', globals())


	def __init__(self, id, name, description=''):
		"Initialize the class instance"

		self.id = id
		self.name = name
		self.description = description

		try:

			self.index_object()

		except AttributeError:

			# No ZCatalog instance available..

			pass

	def get_number_of_files(self):
		"""Returns the number of files this container has."""

		return len(self.objectValues('PersonalFile'))

	def get_number_of_containers(self):
		"""Returns the number of folders this folder has."""

		return len(self.objectValues('PersonalContainer'))


	def get_personal_containers(self):
		"""Returns the personal containers contained."""

		return self.objectValues('PersonalContainer')


default__class_init__(personal_container)

