###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.2 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import Products
import base_item
import string
import smtplib # FIXME
import category

manage_add_todo_folder_form = HTMLFile('todo_folder_add', globals())

def manage_add_todo_folder(self, id, title, number_of_todos_in_display=2,
				REQUEST=None, RESPONSE=None):
	"""Add an instance of the TodoFolder class."""

	self._setObject(id, todo_folder(id, title, number_of_todos_in_display))

	if RESPONSE is not None:

		RESPONSE.redirect(REQUEST['URL3'])

	else:

		return self[id]


class todo_folder(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'TodoFolder'
	icon = 'misc_/TodoFolder/Folder_icon.gif'
	#default_catalog = 'Catalog'

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'number_of_messages_in_display', 'type':'int'}
		)

	meta_types = (
		{'name':'Category', 'action':'manage_add_category_form'},
		)

	manage_add_category_form = category.manage_add_category_form
	manage_add_category = category.manage_add_category

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add Category',
		 ('manage_add_category', 'manage_add_category_form')),
		('View TodoFolders',
		 ('',)),
		)

	index_html = HTMLFile('todo_folder_index', globals())
	display = HTMLFile('todo_folder_display', globals())

	base_edit = HTMLFile('todo_folder_edit', globals())

	product_name = 'TodoFolder'

	def __init__(self, id, title='', number_of_todos_in_display=2):
		"""Initialize the class instance,  and add default forums."""

		self.id = id
		self.title = title
		self.number_of_todos_in_display = number_of_todos_in_display

		self.counter = 0

		try:

			self.index_object()

		except AttributeError:

			# ZCatalog isn't available

			pass

		self.manage_add_category(id='-1', title='Misc',
				description='This category contains miscellaneous todos')
		self.manage_add_category(id='-2', title='Work', 
				description='This container contains work todos')

		self.manage_addDTMLMethod('standard_html_header', 'Helpful navigation and such',
						HTMLFile('standard_html_header', globals()))
		self.manage_addDTMLMethod('standard_html_footer', '',
						HTMLFile('standard_html_footer', globals()))


	def get_sorted_todos(self, number_of_todo_items=None):
		"""Returns a list of todo items sorted, if the maximum number
		of todo items is specified, it will return the most recent items."""

		todo_items = self.get_todos()

		todo_items.sort(
			lambda x,y: cmp(
			y.priority, x.priority
			))

		if number_of_todo_items:

			return todo_items[:number_of_todo_items]

		else:

			return todo_items


	def get_todos(self, number_of_todo_items=10):
		"""Returns all todos."""

		todo_items = []

		for category in self.objectValues('Category'):

			map(todo_items.append, category.get_todos())

		if number_of_todo_items:

			return todo_items[:number_of_todo_items]

		else:

			return todo_items


	def get_display_todos(self):
		"""Returns a list of todo items for the display."""

		return self.get_sorted_todos(self.number_of_todos_in_display)


	def get_categories(self):
		"""Returns all categories contained."""

		return self.objectValues('Category')


default__class_init__(todo_folder)




