###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################


__doc__ = """Workspace manager, to easility manage settings and shares for multiple users."""
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import Products
import product_base_item
import base_item

manage_add_workspace_manager_form = HTMLFile('workspace_manager_add', globals())


def manage_add_workspace_manager(self, id, title, description,
				install_default_html=None,
				install_default_theme=None,
				install_default_id_generator=None,
				REQUEST=None):
	"""Add an instance of the bookmark_folder class."""

	self._setObject(id, workspace_manager(id, title, description,
				install_default_html=install_default_html,
				install_default_theme=install_default_theme,
				install_default_id_generator=install_default_id_generator))

	if REQUEST is not None:

		REQUEST.RESPONSE.redirect(self.absolute_url())


class workspace_manager(
	Products.ZCatalog.CatalogAwareness.CatalogAware,
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	product_base_item.product_base_item,
	base_item.base_item
	):
	"""Used to manage workspaces and shares."""

	meta_type = 'WorkspaceManager'
	icon = 'misc_/WorkspaceManager/Folder_icon.gif'
	default_catalog = 'Catalog'
	product_name = 'WorkspaceManager'

	use_local_navigation = 1
	use_breadcrumbs = 1
	use_all_meta_types = 1

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'description', 'type':'text'},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View Workspaces',
		 ('',))
		)

	# FIXME
	#
	# Can we assign the Manager to do this instead?
	#__ac_roles__ = ('workspace_manager',)

	index_html = HTMLFile('workspace_manager_index', globals())


	def get_workspace_manager(self):

		return self


	def get_bookmark_folder(self):
		"""Returns the 'global' bookmark folder."""

		return self.get_bookmark_folder()


	def get_bookmark_folders(self):
		"""Returns all bookmark folders at this level."""

		return self.objectValues('BookmarkFolder')


	def get_address_book(self):
		"""Returns the 'global' address book."""

		try:

			return self.objectValues('AddressBook')[0]

		except IndexError:

			return None


	def get_address_books(self):
		"""Returns the 'global' address books."""

		return self.objectValues('AddressBook')


	def get_calendar(self):
		"""Returns the 'global' calendar."""

		try:

			return self.objectValues('CalendarFolder')[0]

		except IndexError:

			return None


	def get_calendars(self):
		"""Returns all calendars available."""

		return self.objectValues('CalendarFolder')


	def get_personal_folder(self):
		"""Returns the 'global' personal folder.

		Bit of a contradiction, huh?  :-)"""

		try:

			return self.objectValues('PersonalFolder')[0]

		except IndexError:

			return None


	def get_todo_folder(self):
		"""Returns the todo folder."""

		try:

			return self.objectValues('TodoFolder')[0]

		except IndexError:

			return None


	def get_news_host(self):
		"""Returns the newshost."""

		try:

			return self.objectValues('NewsHost')[0]

		except IndexError:

			return None


	def get_workspace_actions(self, REQUEST=None):
		"""Returns a list of workspace product jumps."""

		actions = []

		calendar = self.get_calendar()

		if calendar:

			actions.append(
				{
				'action':calendar.get_today().relative_url(),
				'title':'Global Calendar'
				})

		address_book = self.get_address_book()

		if address_book:

			actions.append(
				{
				'action':address_book.relative_url(),
				'title':'Global Contacts'
				})

		bookmarks = self.get_bookmark_folder()

		if bookmarks:

			actions.append(
				{
				'action':bookmarks.relative_url(),
				'title':'Global Bookmarks'
				})

		return actions


	def get_workspace_create_actions(self):
		"""Returns a list of 'jump-actions'."""

		action_marker = '' #'- - - - '

		actions = []

		gum = self.get_gum()

		calendar = self.get_calendar()

		if calendar:

			actions.append(
				{
				'action':calendar.relative_url() + '/calendar_folder_add_event',
				'title':action_marker + 'New global event'
				})

			actions.append(
				{
				'action':calendar.relative_url() + '/calendar_folder_add_todo',
				'title':action_marker + 'New global todo'
				})

		address_book = self.get_address_book()

		if address_book:

			actions.append(
				{
				'action':address_book.relative_url() + \
					'/manage_add_address_book_contact_form',
				'title':action_marker + 'New global contact'
				})

		bookmark_folder = self.get_bookmark_folder()

		if bookmark_folder:

			actions.append(
				{
				'action':bookmark_folder.relative_url() + '/manage_add_bookmark_form',
				'title':'New global bookmark'
				})

		return actions


	def add_contact(self, firstname='', surname='', email=''):
		"""Adds a contact in the address book in the 'global' contact folder."""

		address_book = self.get_address_book()

		if address_book is None: return

		new_contact = address_book.manage_add_address_book_contact(firstname=firstname,
					surname=surname, email=email)

		return new_contact


	def get_privilieged_workspace_user_meta_types(self):
		"""Returns a list of users that have can access to to the shared folders."""

		privilieged_users = []

		for object in self.objectValues():

			if getattr(object.aq_base, 'shared_priviliedge', None):

				privilieged_users.append(object.meta_type)


	def __init__(self, id, title='', description='',
			install_default_html=None,
			install_default_theme=None,
			install_default_id_generator=None):
		"""Initialize the class instance."""

		self.id = id
		self.title = title
		self.description = description
		self.use_frames = 1
		self.notifications = []

		product_base_item.utils.default_installation(
				self,
				install_default_html=install_default_html,
				install_default_theme=install_default_theme,
				install_default_id_generator=install_default_theme),

		self.index_object()


default__class_init__(workspace_manager)